@@ -71,7 +71,7 @@ use Digest::MD5;
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '1.4.7';
+our $VERSION = '1.4.8';
 
 use base qw(Lemonldap::NG::Common::CGI Exporter);
 our @ISA;
@@ -483,7 +483,7 @@ sub new {
 # @param ignoreError set to 1 if error should not appear in logs
 # @return boolean
 sub loadModule {
-    my ( $self, $module, $ignoreError ) = splice @_;
+    my ( $self, $module, $ignoreError ) = @_;
 
     return 1 unless $module;
 
@@ -528,7 +528,7 @@ sub getConf {
 # @param base64 Encode value in base64
 # @return nothing
 sub setHiddenFormValue {
-    my ( $self, $key, $val, $prefix, $base64 ) = splice @_;
+    my ( $self, $key, $val, $prefix, $base64 ) = @_;
 
     # Default values
     $prefix = "lmhidden_" unless defined $prefix;
@@ -549,7 +549,7 @@ sub setHiddenFormValue {
 # @param base64 Decode value from base64
 # @return string The associated value
 sub getHiddenFormValue {
-    my ( $self, $key, $prefix, $base64 ) = splice @_;
+    my ( $self, $key, $prefix, $base64 ) = @_;
 
     # Default values
     $prefix = "lmhidden_" unless defined $prefix;
@@ -573,7 +573,7 @@ sub getHiddenFormValue {
 # @param keys Array reference of keys
 # @return nothing
 sub clearHiddenFormValue {
-    my ( $self, $keys ) = splice @_;
+    my ( $self, $keys ) = @_;
 
     unless ( defined $keys ) {
         delete $self->{portalHiddenFormValues};
@@ -637,7 +637,7 @@ sub initCaptcha {
 # @param captcha code generated by Authen::Captcha
 # @return a constant
 sub checkCaptcha {
-    my ( $self, $code, $ccode ) = splice @_;
+    my ( $self, $code, $ccode ) = @_;
 
     # Get captcha object
     my $captcha = Lemonldap::NG::Common::Captcha->new(
@@ -667,7 +667,7 @@ sub checkCaptcha {
 # @param captcha code generated by Authen::Captcha
 # @return a constant
 sub removeCaptcha {
-    my ( $self, $ccode ) = splice @_;
+    my ( $self, $ccode ) = @_;
 
     # Get captcha object
     my $captcha = Lemonldap::NG::Common::Captcha->new(
@@ -696,7 +696,7 @@ sub removeCaptcha {
 # @param value Parameter value
 # @return 1 if url can be trusted, 0 else
 sub isTrustedUrl {
-    my ( $self, $url ) = splice @_;
+    my ( $self, $url ) = @_;
     return
          $url =~ m#^https?://$self->{reVHosts}(:\d+)?/#o
       || $self->{trustedDomains} eq "*"
@@ -710,7 +710,7 @@ sub isTrustedUrl {
 # @param value Parameter value
 # @return 1 if attack detected, 0 else
 sub checkXSSAttack {
-    my ( $self, $name, $value ) = splice @_;
+    my ( $self, $name, $value ) = @_;
 
     # Empty values are not bad
     return 0 unless $value;
@@ -889,14 +889,12 @@ sub getApacheSession {
 # @param id session reference
 # return Lemonldap::NG::Common::Session object
 sub getPersistentSession {
-    my ( $self, $id ) = splice @_;
+    my ( $self, $id ) = @_;
 
     my $persistentSession = Lemonldap::NG::Common::Session->new(
         {
             storageModule        => $self->{persistentStorage},
             storageModuleOptions => $self->{persistentStorageOptions},
-            cacheModule          => $self->{localSessionStorage},
-            cacheModuleOptions   => $self->{localSessionStorageOptions},
             id                   => $id,
             force                => 1,
             kind                 => "Persistent",
@@ -915,7 +913,7 @@ sub getPersistentSession {
 # @param $s String to hash
 # @return hashed value
 sub _md5hash {
-    my ( $self, $s ) = splice @_;
+    my ( $self, $s ) = @_;
     return substr( Digest::MD5::md5_hex($s), 0, 32 );
 }
 
@@ -928,7 +926,7 @@ sub _md5hash {
 # @param id optional SSO session ID
 # @return nothing
 sub updatePersistentSession {
-    my ( $self, $infos, $uid, $id ) = splice @_;
+    my ( $self, $infos, $uid, $id ) = @_;
 
     # Return if no infos to update
     return () unless ( ref $infos eq 'HASH' and %$infos );
@@ -963,7 +961,7 @@ sub updatePersistentSession {
 # @param id Session ID
 # @return nothing
 sub updateSession {
-    my ( $self, $infos, $id ) = splice @_;
+    my ( $self, $infos, $id ) = @_;
 
     # Return if no infos to update
     return () unless ( ref $infos eq 'HASH' and %$infos );
@@ -1010,7 +1008,7 @@ sub updateSession {
 # @param value Value to add
 # @param id optional Session identifier
 sub addSessionValue {
-    my ( $self, $key, $value, $id ) = splice @_;
+    my ( $self, $key, $value, $id ) = @_;
 
     # Mandatory parameters
     return () unless defined $key;
@@ -1048,7 +1046,7 @@ sub addSessionValue {
 # @param value the complete value
 # @return first value
 sub getFirstValue {
-    my ( $self, $value ) = splice @_;
+    my ( $self, $value ) = @_;
 
     my @values = split /\Q$self->{multiValuesSeparator}\E/, $value;
 
@@ -1125,7 +1123,7 @@ sub get_user {
 # @param type auth/user/password/issuer
 # @return module name
 sub get_module {
-    my ( $self, $type ) = splice @_;
+    my ( $self, $type ) = @_;
 
     if ( $type =~ /auth/i ) {
         if ( defined $self->{_multi}->{stack}->[0] ) {
@@ -1346,7 +1344,7 @@ sub stamp {
 # @param $sec number of seconds
 # @return a formated time
 sub convertSec {
-    my ( $self, $sec ) = splice @_;
+    my ( $self, $sec ) = @_;
     my ( $day, $hrs, $min ) = ( 0, 0, 0 );
 
     # Calculate the minutes
@@ -1375,7 +1373,7 @@ sub convertSec {
 # Return skin name
 # @return skin name
 sub getSkin {
-    my ($self) = splice @_;
+    my ($self) = @_;
 
     my $skin = $self->{portalSkin};
 
@@ -1994,6 +1992,8 @@ sub setSessionInfo {
 #@return Lemonldap::NG::Portal constant
 sub setMacros {
     my $self = shift;
+    $self->{sessionInfo}->{groups} = '';
+    $self->{sessionInfo}->{hGroups} = {};
     foreach ( sort keys %{ $self->{macros} } ) {
         $self->{sessionInfo}->{$_} =
           $self->safe->reval( $self->{macros}->{$_} );
