@@ -15,7 +15,7 @@ use Unicode::String qw(utf8);
 use strict;
 
 our @EXPORT   = qw(ldap);
-our $VERSION  = '1.4.1';
+our $VERSION  = '1.4.8';
 our $ppLoaded = 0;
 
 BEGIN {
@@ -114,6 +114,21 @@ sub bind {
     return $mesg;
 }
 
+## @method Net::LDAP::Message unbind()
+# Reimplementation of Net::LDAP::unbind() to force call to disconnect()
+# @return Net::LDAP::Message
+sub unbind {
+    my $self = shift;
+    my $ldap_uri = $self->uri;
+
+    $self->{portal}->lmLog("Unbind and disconnect from $ldap_uri", 'debug');
+
+    my $mesg = $self->SUPER::unbind();
+    $self->SUPER::disconnect();
+
+    return $mesg;
+}
+
 ## @method private boolean loadPP ()
 # Load Net::LDAP::Control::PasswordPolicy
 # @return true if succeed.
@@ -146,6 +161,7 @@ sub loadPP {
 # @return Lemonldap::NG portal error code
 sub userBind {
     my $self = shift;
+
     if ( $self->{portal}->{ldapPpolicyControl} ) {
 
         # Create Control object
@@ -318,9 +334,33 @@ sub userModifyPassword {
 
                     # Check old password with a bind
                     $mesg = $self->bind( $dn, password => $oldpassword );
-                    if ( $mesg->code != 0 ) {
-                        $self->{portal}->lmLog( "Bad old password", 'debug' );
-                        return PE_BADOLDPASSWORD;
+
+                    # For AD password expiration to work:
+                    # ppolicy must be desactivated,
+                    # and "change as user" must be desactivated
+                    if ($ad) {
+                        if ( $mesg->error =~ /LdapErr: .* data ([^,]+),.*/ ) {
+
+# extended data message code:
+# 532: password expired (but provided password is correct)
+# 773: must change password at next connection (but provided password is correct)
+# 52e: password is incorrect
+                            unless ( ( $1 eq '532' ) || ( $1 eq '773' ) ) {
+                                $self->{portal}
+                                  ->lmLog( "Bad old password", 'warn' );
+                                return PE_BADOLDPASSWORD;
+                            }
+                        }
+
+                   # if error message has not been catched, then it IS a success
+                    }
+                    else
+                    {   # this is not AD, a 0 error code means good old password
+                        if ( $mesg->code != 0 ) {
+                            $self->{portal}
+                              ->lmLog( "Bad old password", 'warn' );
+                            return PE_BADOLDPASSWORD;
+                        }
                     }
 
           # Rebind as Manager only if user is not granted to change its password
@@ -493,12 +533,12 @@ sub ldap {
 # @param key Attribute name in group containing searched value
 # @param value Searched value
 # @param attributes to get from found groups (array ref)
-# @return string groups separated with multiValuesSeparator
+# @return hashRef groups
 sub searchGroups {
     my ( $self, $base, $key, $value, $attributes ) = splice @_;
 
     my $portal = $self->{portal};
-    my $groups;
+    my $groups = {};
 
     # Creating search filter
     my $searchFilter =
@@ -543,30 +583,31 @@ sub searchGroups {
                       $self->searchGroups( $base, $key, $group_value,
                         $attributes );
 
-                    $groups .=
-                      $recursive_groups . $portal->{multiValuesSeparator}
-                      if ($recursive_groups);
+                    my %allGroups = ( %$groups, %$recursive_groups )
+                      if ( ref $recursive_groups );
+                    $groups = \%allGroups;
                 }
             }
 
+            # Use first attribute as group name
+            my $groupName = $entry->get_value( $attributes->[0] );
+            $groups->{$groupName}->{name} = $groupName;
+
             # Now parse attributes
             foreach (@$attributes) {
 
                 # Next if group attribute value
                 next if ( $_ eq $portal->{ldapGroupAttributeValueGroup} );
 
-                my $data = $entry->get_value($_);
+                my $data = $entry->get_value( $_, asref => 1 );
 
                 if ($data) {
-                    $portal->lmLog( "Store $data in groups", 'debug' );
-                    $groups .= $data . "|";
+                    $portal->lmLog( "Store values of $_ in group $groupName",
+                        'debug' );
+                    $groups->{$groupName}->{$_} = $data;
                 }
             }
-            $groups =~ s/\|$//g;
-            $groups .= $portal->{multiValuesSeparator};
         }
-
-        $groups =~ s/\Q$portal->{multiValuesSeparator}\E$//;
     }
 
     return $groups;
